import * as Config from "./modules/Config";
import * as Server from "./modules/Server";
import Log from "./modules/Log";
import UserProperty from "./modules/UserProperty";
import * as LocalData from "./modules/LocalData";
import * as Utils from './modules/Utils';
import  ActionQueue from './modules/ActionQueue'
import Const from './modules/Const';

const PRE_INIT_ACTIONS = ['setAppKey', 'setAutoPageView'];
let g_instance = null;
export default class App {
    constructor() {
        this._autoPageView = true;
        this._actionQueue = new ActionQueue();
    }

    start() {
        this._processFunctionCall();

        if (this._autoPageView) {
            this.pushAction('trackPageView');
        }

        this._loop();
    };

    pushAction(actionName, ...args) {
        this._actionQueue.push([actionName, ...args]);
    }

    _loop() {

        this._processFunctionCall();

        const self = this;

        setTimeout(function () {
            self._loop();
        }, 50);
    };

    _processFunctionCall() {
        let actions;
        if (!Config.isAppKeyValid()) {
            actions = this._actionQueue.popAllMatchTypes(PRE_INIT_ACTIONS);
        }
        else {
            actions = this._actionQueue.popAll();
        }
        while (actions.length > 0) {
            /**
             * @type {Array}
             */
            const item = actions.shift();
            const functionName = item[0];
            const args = item.slice(1);

            if (this[functionName] !== undefined) {
                const f = this[functionName];
                f.apply(this, args);
            }
            else {
                console.warn("function not exist:" + functionName + " for" + item.join(","));
            }
        }
    };

    setAppKey(appKey) {
        Config.setAppKey(appKey);
    };

    /**
     * @param autoPageView bool
     */
    setAutoPageView(autoPageView) {
        this._autoPageView = autoPageView;
    };

    /**
     * @param url
     * @param referrer
     */
    trackPageView(url, referrer) {
        if (Utils.RUNTIME === Utils.RUNTIME_TYPE.BROWSER) {
            url = url || window.location.href;
            referrer = referrer || document.referrer;
        }

        Server.sendPV(url, referrer);
    };

    trackEvent(category, action, label, value) {
        Server.sendAction(category, action, label, value);
    };

    sendEvent(eventId, params) {
        Server.sendEvent(eventId, params);
    }

    setLogHandler(logHandler) {
        Log.getInstance().setLogHandler(logHandler);
    };

    actionTimeStart(action, extdata) {
        Server.actionTimeStart("custom", action, extdata)
    };

    actionTimeEnd(action) {
        Server.actionTimeEnd(action)
    };

    setUserId(userId) {
        LocalData.setItem("user_id", userId);
    };

    setChannel(channel) {
        LocalData.setItem("channel", channel);
    };

    setUserProperties(properties) {
        UserProperty.getInstance().setProperties(properties)
    };

    setVersion(version) {
        LocalData.setItem("version", version);
    };

    recActionExpose(item) {
        Server.sendRecommendAction(Const.RECOMMEND_ACTION_EXPOSE, item, 1);
    }

    recActionView(item) {
        Server.sendRecommendAction(Const.RECOMMEND_ACTION_VIEW, item, 1);
    }

    recActionTransact(item) {
        Server.sendRecommendAction(Const.RECOMMEND_ACTION_TRANSACT, item, 1);
    }

    recActionCollect(item) {
        Server.sendRecommendAction(Const.RECOMMEND_ACTION_COLLECT, item, 1);
    }

    recActionUncollect(item) {
        Server.sendRecommendAction(Const.RECOMMEND_ACTION_COLLECT, item, 0);
    }

    recActionLike(item) {
        Server.sendRecommendAction(Const.RECOMMEND_ACTION_LIKE, item, 1);
    }

    recActionDislike(item) {
        Server.sendRecommendAction(Const.RECOMMEND_ACTION_LIKE, item, 0);
    }


    static getInstance() {
        if (g_instance === null) {
            g_instance = new App();
        }
        return g_instance;
    };
}

