let socket = null;
let isOpen = false;
let instanceId = 0;
let instance = null;
let timeout = null;

export default class ActionTime {
    constructor(url) {
        try {
            if (!socket) {
                socket = new WebSocket(url);
                socket.onopen = function () {
                    isOpen = true;
                    timeout = setInterval(function () {
                        socket.send('{"cmd":"action_continue"}')
                    }, 5000)
                };

                socket.onclose = function () {
                    clearInterval(timeout);
                };
            }
        }
        catch (e) {
            console.warn("不支持WebSocket, 无法精确统计时常");
        }

        this.instanceId = instanceId++;
        this.timeout = null;
    };

    start(data) {
        let _this = this;
        if (!isOpen) {
            //处于等待连接成功状态，轮训, 放入时间队列
            setTimeout(function () {
                _this.start(data);
            }, 300);
        }
        else {
            socket.send(JSON.stringify({cmd: 'action_start', params: data}));
        }
    }

    end(data) {
        const _this = this;
        if (!isOpen) {
            //处于等待连接成功状态，轮训, 放入时间队列
            setTimeout(function () {
                _this.end(data)
            }, 300)
        } else {
            socket.send(JSON.stringify({cmd: 'action_end', params: data}))
        }
    }

    static getInstance(url) {
        if (instance) {
            return instance;
        }

        instance = new ActionTime(url);

        return instance;
    }
}

