import * as EventStorage from './EventStorage';
import * as Net from './Net';
import * as Config from './Config';
import BuildConfig from './BuildConfig';


const SEND_INTERVAL = 5 * 1000;
const MAX_BATCH_SIZE = 50;

let sending = false;

const allSender = {};

class EventSender {
    constructor(type, sender) {
        this._type = type;
        this._storage = EventStorage.get(type);
        this._sender = sender;
    }

    send(data) {
        this._storage.push(data);
    }

    _tick(cb) {
        function next() {
            setTimeout(cb, 1);
        }

        const batch_size = Math.min(MAX_BATCH_SIZE, this._storage.length);

        if (batch_size <= 0) {
            return next();
        }

        const batch = this._storage.shift(batch_size);

        const that = this;
        this._sender(batch, function () {
            next();

        }, function () {
            next();
        })
    }
}


function start_timer() {
    function timer_tick(cb) {
        let senders = [];
        for (let k in allSender) {
            senders.push(allSender[k]);
        }

        function sender_tick(i) {
            if (i >= senders.length) {
                cb();
                return;
            }

            let sender = senders[i];
            sender._tick(function () {
                sender_tick(i + 1);
            })
        }

        sender_tick(0);
    }

    timer_tick(function () {
        setTimeout(start_timer, SEND_INTERVAL);
    })
}

start_timer();

/**
 * @param type
 * @param sender
 * @returns {EventSender}
 */
function create_sender(type, sender) {
    if (!allSender[type]) {
        allSender[type] = new EventSender(type, sender);
    }

    return allSender[type];
}

/**
 * @type {EventSender}
 */
export const userEvent = create_sender("user", function (batch, success, fail) {
    Net.send(BuildConfig.SERVER + "/api/web/event", {"events": batch}, function () {
        success();

    }, function () {
        fail();
    })
});

/**
 * @type {EventSender}
 */
export const recommendEvent = create_sender("rec", function (batch, success, fail) {
    Net.send(BuildConfig.SERVER + "/api/recommend/event", {"events": batch}, function () {
        success();

    }, function () {
        fail();
    })
});
