import * as Utils from './Utils';

/**
 * @param url
 * @param data
 * @param onSuccess
 * @param onError
 */
export function send(url, data, onSuccess, onError) {
    if (Utils.RUNTIME === Utils.RUNTIME_TYPE.BROWSER) {
        const xmlHttpRequest = new XMLHttpRequest();
        xmlHttpRequest.onreadystatechange = function () {
            if (xmlHttpRequest.readyState === 4) {
                if (xmlHttpRequest.status === 200) {
                    if (typeof onSuccess === "function") {
                        onSuccess(xmlHttpRequest.responseText);
                    }
                }
                else {
                    if (typeof onError === "function") {
                        onError();
                    }
                }
            }
        };

        xmlHttpRequest.open("POST", url, true);
        xmlHttpRequest.send(JSON.stringify(data));
    }
    else if (Utils.RUNTIME === Utils.RUNTIME_TYPE.WX_APP) {
        wx.request({
            url: url,
            data: data,
            method: 'POST',
            header: {'content-type': 'application/json'},
            success(res) {
                if (typeof onSuccess === 'function') {
                    onSuccess(JSON.stringify(res.data));
                }
            },
            fail() {
                if (typeof onError === "function") {
                    onError();
                }
            }
        });
    }
    else {
        console.error("unsupported platform")
    }
}
