import * as LocalData from "./LocalData";
import UserProperty from "./UserProperty";
import * as Config from "./Config";
import * as Net from "./Net";
import * as EventSender from "./EventSender";
import ActionTime from "./ActionTime";
import * as EventId from "./EventId";
import * as Utils from './Utils';
import * as DeviceId from './DeviceId';
import BuildConfig from './BuildConfig';
import * as Session from './Session';
const USER_ID_KEY = "user_id";


function processDeviceId(result) {
    try {
        const result_obj = JSON.parse(result);
        const uid = result_obj["uid"];
        if (!Utils.isStringEmpty(uid)) {
            DeviceId.setDeviceId(uid);
        }
    }
    catch (e) {
        console.error(e);
    }
}

function appendDeviceId(data) {
    let uid = DeviceId.getDeviceId();

    data["uid"] = uid;
    data["device_id"] = uid;
}

function appendVersionInfo(data) {
    data["version"] = Config.getVersion();
}

function appendInfo(data) {
    data["id"] = EventId.next();
    data["time"] = new Date().getTime();
    data['platform'] = Utils.PLATFORM;
}

function appendUserInfo(data) {
    data["user_properties"] = UserProperty.getInstance().getProperties();
    let channel = LocalData.getItem("channel");
    if (channel === null) {
        channel = "";
    }
    let userId = LocalData.getItem(USER_ID_KEY);
    if (userId === null) {
        userId = "";
    }

    let version = LocalData.getItem("version");
    if (version === null) {
        version = "";
    }

    data["channel"] = channel;
    data["user_id"] = userId;
    data["version"] = version;
    data["sdk_version"] = BuildConfig.SDK_VERSION;
}

export function sendPV(url, referrer) {
    const data = {appkey: Config.getAppKey(), url: url, referrer: referrer};

    appendDeviceId(data);

    appendVersionInfo(data);

    appendInfo(data);

    appendUserInfo(data);

    Net.send(BuildConfig.SERVER + "/api/web/pv", data, function (result) {
        processDeviceId(result);
    }, null);
}

export function sendRecommendUserAction(actionData) {
    const data = {appkey: Config.getAppKey(), ...actionData};
    // const data= {};

    appendInfo(data);
    appendUserInfo(data);
    EventSender.recommendEvent.send(data);
}

export function sendAction(category, action, label, value) {
    const data = {appkey: Config.getAppKey(), category: category, action: action, label: label, value: value};

    appendDeviceId(data);

    appendVersionInfo(data);

    appendInfo(data);

    appendUserInfo(data);

    EventSender.userEvent.send(data);
}

export function sendEvent(eventId, params) {
    const data = {appkey: Config.getAppKey(), action: eventId, params};

    appendDeviceId(data);

    appendVersionInfo(data);

    appendInfo(data);

    appendUserInfo(data);

    EventSender.userEvent.send(data);
}


export function actionTimeStart(type, action, extdata) {
    const data = {app_id: Config.getAppKey(), type: type, event_id: action, params: extdata};

    appendDeviceId(data);

    appendVersionInfo(data);

    appendInfo(data);

    appendUserInfo(data);

    const actionTime = ActionTime.getInstance(BuildConfig.WS_SERVER);

    actionTime.start(data)
}

export function actionTimeEnd(action) {
    const data = {event_id: action};

    const actionTime = ActionTime.getInstance(BuildConfig.WS_SERVER);

    actionTime.end(data)
}

export function sendRecommendAction(action, item, weight) {
    let user_id = LocalData.getItem(USER_ID_KEY);
    if (Utils.isStringEmpty(user_id)) {
        user_id = "";
    }

    EventSender.recommendEvent.send({
        app_id: Config.getAppKey(),
        user_id,
        item_id: item,
        act_type: action,
        session_id: Session.getId(),
        weight
    });
}

