import IStorage from './IStorage';

function localStorageGetItem(key) {
    if (!window.localStorage) {
        return "";
    }

    return window.localStorage.getItem(key);
}

function localStorageSetItem(key, value) {
    if (!window.localStorage) {
        return;
    }

    window.localStorage.setItem(key, value);
}

function setCookie(name, value, expiredays) {
    const ex_data = new Date();
    ex_data.setDate(ex_data.getDate() + expiredays);
    //noinspection JSDeprecatedSymbols
    document.cookie = name + "=" + escape(value) +
        ((expiredays === null) ? "" : ";expires=" + ex_data.toUTCString());
}

function getCookie(c_name) {
    if (document.cookie.length > 0) {
        let c_start = document.cookie.indexOf(c_name + "=");
        if (c_start !== -1) {
            c_start = c_start + c_name.length + 1;
            let c_end = document.cookie.indexOf(";", c_start);
            if (c_end === -1) c_end = document.cookie.length;
            //noinspection JSDeprecatedSymbols
            return unescape(document.cookie.substring(c_start, c_end));
        }
    }
    return ""
}

export default class BrowserStorage extends IStorage {
    getItem(key) {
        const value = localStorageGetItem(key);
        if (value !== null && value !== "") {
            return value;
        }

        return getCookie(key);
    }

    setItem(key, value) {
        localStorageSetItem(key, value);
        setCookie(key, value, 365);
    }
}
